@@ -18,12 +18,11 @@ from pytailer import async_fail_tail
 
 #logging.basicConfig(level=logging.DEBUG)
 
-async def tailfile(proc):
-    with async_fail_tail(proc.stdout, lines=10) as tail:
+async def tailfile(f):
+    with async_fail_tail(f, lines=10) as tail:
         async for line in tail:  # be careful: infinite loop!
             print(line, end='', flush=True)
 
-
 def _setup():
     pm3_home_dir = Path('~/.pm3').expanduser()
     config_file = f'{pm3_home_dir}/config.ini'
@@ -63,6 +62,45 @@ def _read_config():
     config.read(config_file)
     return config
 
+def _make_systemd_init_script():
+    config = _read_config()
+    pm3_service = '''#!/bin/bash
+
+cat << EOF > /etc/systemd/system/pm3.service
+[Unit]
+Description=PM3 Backend
+After=network.target
+
+[Service]
+User={}
+Type=simple
+ExecStart={}
+Restart=always
+
+[Install]
+WantedBy=multi-user.target
+EOF
+echo 'file /etc/systemd/system/pm3.service written'
+
+echo 'systemctl daemon-reload'
+systemctl daemon-reload
+
+echo 'systemctl enable pm3'
+systemctl enable pm3
+
+echo 'systemctl start pm3'
+systemctl start pm3
+systemctl is-active pm3
+'''
+    backend_start_interpreter = config['main_section'].get('backend_start_interpreter')
+    backend_start_command = config['main_section'].get('backend_start_command')
+    exec_start = f'{backend_start_interpreter} {backend_start_command}' if backend_start_interpreter else backend_start_command
+    with open('systemd.sh', 'w') as f:
+        f.write(pm3_service.format(os.getlogin(), exec_start))
+        print('file systemd.sh generated')
+    print('now execute:\n sudo bash systemd.sh')
+
+
 def _get(path) -> RetMsg:
     config = _read_config()
     base_url = config['main_section'].get('backend_url')
@@ -109,15 +147,18 @@ def _ls(id_or_name='all', format='table'):
 
 def _ps(id_or_name='all', format='table'):
     res = _get(f'ps/{id_or_name}')
-    if res:
-        payload_sorted = sorted(res.payload, key=lambda item: item.get("pm3_id"))
-        if format == 'table':
-            return _tabulate_ps(payload_sorted)
+    if not res.err:
+        if res.payload:
+            payload_sorted = sorted(res.payload, key=lambda item: item.get("pm3_id"))
+            if format == 'table':
+                return _tabulate_ps(payload_sorted)
+            else:
+                payload_sorted = [ProcessStatus(**p).dict() for p in payload_sorted]
+                return _show_list(payload_sorted)
         else:
-            payload_sorted = [ProcessStatus(**p).dict() for p in payload_sorted]
-            return _show_list(payload_sorted)
+            return '[yellow]there is nothing to look at[/yellow]'
     else:
-        return '[yellow]there is nothing to look at[/yellow]'
+        return f'[red]{res.msg}[/red]'
 
 def _show_list(data):
     out = []
@@ -126,7 +167,7 @@ def _show_list(data):
         out.append(f"[cyan]### {row['pm3_name']} ({row['pm3_id']}) ###[/cyan]")
         for k, v in row.items():
             out.append(f'  {k}={v}')
-        return '\n'.join(out)
+    return '\n'.join(out)
 
 def _tabulate_ps(data):
     if len(data) == 0:
@@ -200,6 +241,13 @@ def main():
     parser = argparse.ArgumentParser(prog='pm3', description='Like pm2 without node.js')
     subparsers = parser.add_subparsers(dest='subparser')
 
+    parser_daemon = subparsers.add_parser('daemon', help='Daemon options')
+    parser_daemon.add_argument('what', default='status', const='status', nargs='?',
+                               choices=['start', 'stop', 'status', 'systemd'])
+
+    parser_ping = subparsers.add_parser('ping', help='Ensure pm3 daemon has been launched')
+    parser_ping.add_argument('-v', '--verbose', action='store_true', help='verbose')
+
     parser_ls = subparsers.add_parser('ls', help='process list')
     parser_ls.add_argument('id_or_name', const='all', nargs='?', type=str, help='id or process name')
     parser_ls.add_argument('-l', '--list', action='store_true', help='List format')
@@ -208,12 +256,6 @@ def main():
     parser_ps.add_argument('id_or_name', const='all', nargs='?', type=str, help='id or process name')
     parser_ps.add_argument('-l', '--list', action='store_true', help='List format')
 
-    parser_daemon = subparsers.add_parser('daemon', help='Daemon options')
-    parser_daemon.add_argument('what', default='status', const='status', nargs='?', choices=['start', 'stop', 'status'])
-
-    parser_ping = subparsers.add_parser('ping', help='Ensure pm3 daemon has been launched')
-    parser_ping.add_argument('-v', '--verbose', action='store_true', help='verbose')
-
     parser_new = subparsers.add_parser('new', help='create a new process')
     parser_new.add_argument('cmd', help='linux command')
     parser_new.add_argument('--cwd', dest='cwd', help='cwd of executable file')
@@ -224,6 +266,7 @@ def main():
     parser_new.add_argument('--autorun', dest='pm3_autorun', action='store_true', help='process autorun after reboot')
     parser_new.add_argument('--stdout', dest='pm3_stdout', help='std out')
     parser_new.add_argument('--stderr', dest='pm3_stderr', help='std err')
+    parser_new.add_argument('--interpreter', dest='interpreter', help='interpreter path')
 
     parser_start = subparsers.add_parser('start', help='start a process by id or name')
     parser_start.add_argument('id_or_name', help='Start id or process name')
@@ -304,6 +347,9 @@ def main():
         if args.what == 'status':
             print(_ps(0, 'list'))
 
+        if args.what == 'systemd':
+            _make_systemd_init_script()
+
     elif args.subparser == 'ls':
         id_or_name = args.id_or_name or 'all'
         format_ = 'list' if args.list else 'table'
@@ -319,6 +365,7 @@ def main():
                     cwd=args.cwd or Path.home().as_posix(),
                     pm3_name=args.pm3_name or '',
                     pm3_id=args.pm3_id or -1,
+                    interpreter=args.interpreter or '',
                     nohup=args.nohup,
                     shell=args.pm3_shell,
                     autorun=args.pm3_autorun,
@@ -334,7 +381,6 @@ def main():
         res = _get(f'rm/{args.id_or_name}')
         _parse_retmsg_payload(res)
 
-
     elif args.subparser == 'start':
         res = _get(f'start/{args.id_or_name}')
         _parse_retmsg_payload(res)
@@ -347,26 +393,25 @@ def main():
         res = _get(f'restart/{args.id_or_name}')
         _parse_retmsg_payload(res)
 
-    elif args.subparser == 'restart2':
-        res = _get(f'stop/{args.id_or_name}')
-        if res.err:
-            print(f"[red]{res.msg}[/red]")
-        else:
-            res = _get(f'start/{args.id_or_name}')
-            if res.err:
-                print(f"[red]{res.msg}[/red]")
-            else:
-                print(f"[green]{res.msg}[/green]")
-
     elif args.subparser == 'reset':
         res = _get(f'reset/{args.id_or_name}')
         _parse_retmsg_payload(res)
 
-    #elif args.subparser == 'log':
-        #ion = find_id_or_name(tbl, args.id_or_name)
-        #print(ion)
-        #if len(ion.proc) == 1:
-        #    asyncio.run(tailfile(ion.proc[0]))
+    elif args.subparser in ('log', 'err'):
+        res = _get(f'ls/{args.id_or_name}')
+        if res and not res.err:
+            for p in res.payload:
+                #print(p)
+                if args.subparser == 'log':
+                    ftt = p['stdout']
+                else:
+                    ftt = p['stderr']
+
+                try:
+                    asyncio.run(tailfile(ftt))
+                except KeyboardInterrupt:
+                    #print('CTRL+C')
+                    pass
 
     else:
         print(parser.format_help())
